#!/etc/opt/openit/perl -w

use strict;
use Cwd;

# Only root can patch
if ( $< ne 0 ) {
    die "Error: Patching must be done as 'root'!\n";
}

# Check for architecture
my $arch = `cut -d " " -f 1 /var/opt/openit/inst/product/base.prod`;
chomp $arch;
if( ! defined $arch ) {
    print "ERROR: No architecture defined.\n";
    exit 1;
}

if( $arch ne "i686-redhat-linux" &&
    $arch ne "x86_64-unknown-linux" &&
    $arch ne "sparc-sun-solaris2.8" &&
    $arch ne "powerpc-ibm-aix5.1.0.0" ) {
    print "Unsupported server platform: $arch\n";
    exit 1;
}

print "Installing perl for '$arch'\n";

# Verify that installed version is different.
my $inst_arch = `. /etc/opt/openit/openit.env 2>/dev/null; echo \$OPENIT_ARCH`;
chomp($inst_arch);

if( ! defined $inst_arch || $inst_arch =~ m/^\s*$/ ) {
    print "Failed to get installation architecture.\n";
    exit 1;
}

&change_permissions();

if ($inst_arch eq $arch &&
    -f "/opt/openit/libexec/$inst_arch/lib/perl5/site_perl/5.8.8/JSON.pm") {
    print "Perl with JSON for '$inst_arch' already installed\n";
    exit 0;
}

my $arch_dir = "/opt/openit/libexec/$arch";

# find old perl
my @perl = <$arch_dir/bin/perl?.?.?>;
if( scalar @perl != 1 ) {
    print "Can not identify Perl.\n";
    exit 1;
}
my $perl = shift @perl;
chomp $perl;

# rename old perl...
print "Renaming '$perl'\n";
unlink "$perl.tmp" if -f "$perl.tmp";
rename "$perl", "$perl.tmp" or die "$!\n";
print "Removing '$perl.tmp'\n";
unlink( "$perl.tmp" ) or die "$!\n";

# ... and its link
$perl =~ s#perl.....#perl#;
print "Renaming '$perl'\n";
unlink "$perl.tmp" if -f "$perl.tmp";
rename "$perl", "$perl.tmp" or die "$!\n";
print "Removing '$perl.tmp'\n";
unlink( "$perl.tmp" ) or die "$!\n";


# Unpack tar archive
my $tarf = getcwd() . "/files/$arch/perl.tar.gz";
if (! chdir $arch_dir) {
    print "Unable to chdir to '$arch_dir'\n";
    exit 4;
}
if( ! -f $tarf ) {
    print "Unable to find $tarf\n";
    exit 5;
}
my $command = "/opt/openit/bin/tar pxfz $tarf";
print "Running: $command\n";
exec $command; # exec is used to avoid seg-fault

sub change_permissions {
    print "Setting permissions of Perl library.\n";
    my $cmd = "find /opt/openit/libexec/$inst_arch/lib/perl5";
    $cmd = "find /opt/openit/libexec/$inst_arch/lib/perl5";
    my @files = `$cmd`;
    foreach my $file( @files ) {
	chomp $file;
	if( -d $file ) {
	    system( "chmod a+x $file" );
	}
	system( "chmod a+r $file" );
    }
}
